package sun.util.locale;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Map;

public final class InternalLocaleBuilder
{
    private static final InternalLocaleBuilder.CaseInsensitiveChar PRIVATEUSE_KEY;
    private String language;
    private String script;
    private String region;
    private String variant;
    private Map<InternalLocaleBuilder.CaseInsensitiveChar, String> extensions;
    private Set<InternalLocaleBuilder.CaseInsensitiveString> uattributes;
    private Map<InternalLocaleBuilder.CaseInsensitiveString, String> ukeywords;
    
    public InternalLocaleBuilder() {
        this.language = "";
        this.script = "";
        this.region = "";
        this.variant = "";
    }
    
    public InternalLocaleBuilder setLanguage(String s) throws LocaleSyntaxException {
        if (LocaleUtils.isEmpty(s)) {
            this.language = "";
        }
        else {
            if (!LanguageTag.isLanguage(s)) {
                throw new LocaleSyntaxException("Ill-formed language: " + s, 0);
            }
            this.language = s;
        }
        return this;
    }
    
    public InternalLocaleBuilder setScript(String s) throws LocaleSyntaxException {
        if (LocaleUtils.isEmpty(s)) {
            this.script = "";
        }
        else {
            if (!LanguageTag.isScript(s)) {
                throw new LocaleSyntaxException("Ill-formed script: " + s, 0);
            }
            this.script = s;
        }
        return this;
    }
    
    public InternalLocaleBuilder setRegion(String s) throws LocaleSyntaxException {
        if (LocaleUtils.isEmpty(s)) {
            this.region = "";
        }
        else {
            if (!LanguageTag.isRegion(s)) {
                throw new LocaleSyntaxException("Ill-formed region: " + s, 0);
            }
            this.region = s;
        }
        return this;
    }
    
    public InternalLocaleBuilder setVariant(String str) throws LocaleSyntaxException {
        if (LocaleUtils.isEmpty(str)) {
            this.variant = "";
        }
        else {
            String replaceAll = str.replaceAll("-", "_");
            int checkVariants = this.checkVariants(replaceAll, "_");
            if (checkVariants != -1) {
                throw new LocaleSyntaxException("Ill-formed variant: " + str, checkVariants);
            }
            this.variant = replaceAll;
        }
        return this;
    }
    
    public InternalLocaleBuilder addUnicodeLocaleAttribute(String str) throws LocaleSyntaxException {
        if (!UnicodeLocaleExtension.isAttribute(str)) {
            throw new LocaleSyntaxException("Ill-formed Unicode locale attribute: " + str);
        }
        if (this.uattributes == null) {
            this.uattributes = new HashSet(4);
        }
        this.uattributes.add(new CaseInsensitiveString(str));
        return this;
    }
    
    public InternalLocaleBuilder removeUnicodeLocaleAttribute(String str) throws LocaleSyntaxException {
        if (str == null || !UnicodeLocaleExtension.isAttribute(str)) {
            throw new LocaleSyntaxException("Ill-formed Unicode locale attribute: " + str);
        }
        if (this.uattributes != null) {
            this.uattributes.remove(new CaseInsensitiveString(str));
            return this;
        }
        return this;
    }
    
    public InternalLocaleBuilder setUnicodeLocaleKeyword(String str, String str2) throws LocaleSyntaxException {
        if (!UnicodeLocaleExtension.isKey(str)) {
            throw new LocaleSyntaxException("Ill-formed Unicode locale keyword key: " + str);
        }
        InternalLocaleBuilder.CaseInsensitiveString caseInsensitiveString = new CaseInsensitiveString(str);
        if (str2 == null) {
            if (this.ukeywords != null) {
                this.ukeywords.remove(caseInsensitiveString);
            }
        }
        else {
            if (str2.length() != 0) {
                StringTokenIterator stringTokenIterator = new StringTokenIterator(str2.replaceAll("_", "-"), "-");
                while (!stringTokenIterator.isDone()) {
                    if (!UnicodeLocaleExtension.isTypeSubtag(stringTokenIterator.current())) {
                        throw new LocaleSyntaxException("Ill-formed Unicode locale keyword type: " + str2, stringTokenIterator.currentStart());
                    }
                    stringTokenIterator.next();
                }
            }
            if (this.ukeywords == null) {
                this.ukeywords = new HashMap(4);
            }
            this.ukeywords.put(caseInsensitiveString, str2);
        }
        return this;
    }
    
    public InternalLocaleBuilder setExtension(char c, String s) throws LocaleSyntaxException {
        int privateusePrefixChar = LanguageTag.isPrivateusePrefixChar(c);
        if (privateusePrefixChar == 0 && !LanguageTag.isExtensionSingletonChar(c)) {
            throw new LocaleSyntaxException("Ill-formed extension key: " + c);
        }
        int empty = LocaleUtils.isEmpty(s);
        InternalLocaleBuilder.CaseInsensitiveChar caseInsensitiveChar = new CaseInsensitiveChar(c);
        if (!(empty == 0)) {
            if (UnicodeLocaleExtension.isSingletonChar(caseInsensitiveChar.value())) {
                if (this.uattributes != null) {
                    this.uattributes.clear();
                }
                if (this.ukeywords != null) {
                    this.ukeywords.clear();
                }
            }
            else if (this.extensions != null && this.extensions.containsKey(caseInsensitiveChar)) {
                this.extensions.remove(caseInsensitiveChar);
            }
        }
        else {
            String replaceAll = s.replaceAll("_", "-");
            StringTokenIterator stringTokenIterator = new StringTokenIterator(replaceAll, "-");
            while (!stringTokenIterator.isDone()) {
                String current = stringTokenIterator.current();
                int n;
                if (!(privateusePrefixChar == 0)) {
                    n = LanguageTag.isPrivateuseSubtag(current);
                }
                else {
                    n = LanguageTag.isExtensionSubtag(current);
                }
                if (n == 0) {
                    throw new LocaleSyntaxException("Ill-formed extension value: " + current, stringTokenIterator.currentStart());
                }
                stringTokenIterator.next();
            }
            if (UnicodeLocaleExtension.isSingletonChar(caseInsensitiveChar.value())) {
                this.setUnicodeLocaleExtension(replaceAll);
            }
            else {
                if (this.extensions == null) {
                    this.extensions = new HashMap(4);
                }
                this.extensions.put(caseInsensitiveChar, replaceAll);
            }
        }
        return this;
    }
    
    public InternalLocaleBuilder setExtensions(String replaceAll) throws LocaleSyntaxException {
        if (LocaleUtils.isEmpty(replaceAll)) {
            this.clearExtensions();
            return this;
        }
        replaceAll = replaceAll.replaceAll("_", "-");
        StringTokenIterator stringTokenIterator = new StringTokenIterator(replaceAll, "-");
        Object o = null;
        String string = null;
        int n = 0;
        while (!stringTokenIterator.isDone()) {
            String current = stringTokenIterator.current();
            if (LanguageTag.isExtensionSingleton(current)) {
                int currentStart = stringTokenIterator.currentStart();
                StringBuilder o2 = new StringBuilder(current);
                stringTokenIterator.next();
                while (!stringTokenIterator.isDone()) {
                    String current2 = stringTokenIterator.current();
                    if (LanguageTag.isExtensionSubtag(current2)) {
                        o2.append("-").append(current2);
                        n = stringTokenIterator.currentEnd();
                        stringTokenIterator.next();
                    }
                    else {
                        break;
                    }
                }
                if (n < currentStart) {
                    throw new LocaleSyntaxException("Incomplete extension '" + current + "'", currentStart);
                }
                if (o == null) {
                    o = new ArrayList(4);
                }
                o.add(o2.toString());
            }
            else {
                break;
            }
        }
        if (!stringTokenIterator.isDone()) {
            String current3 = stringTokenIterator.current();
            if (LanguageTag.isPrivateusePrefix(current3)) {
                int currentStart2 = stringTokenIterator.currentStart();
                StringBuilder o3 = new StringBuilder(current3);
                stringTokenIterator.next();
                while (!stringTokenIterator.isDone()) {
                    String current4 = stringTokenIterator.current();
                    if (!LanguageTag.isPrivateuseSubtag(current4)) {
                        break;
                    }
                    o3.append("-").append(current4);
                    n = stringTokenIterator.currentEnd();
                    stringTokenIterator.next();
                }
                if (n <= currentStart2) {
                    throw new LocaleSyntaxException("Incomplete privateuse:" + replaceAll.substring(currentStart2), currentStart2);
                }
                string = o3.toString();
            }
        }
        if (!stringTokenIterator.isDone()) {
            throw new LocaleSyntaxException("Ill-formed extension subtags:" + replaceAll.substring(stringTokenIterator.currentStart()), stringTokenIterator.currentStart());
        }
        return this.setExtensions(o, string);
    }
    
    private InternalLocaleBuilder setExtensions(List<String> list, String s) {
        this.clearExtensions();
        if (!LocaleUtils.isEmpty(list)) {
            HashSet set = new HashSet(list.size());
            for (String s2 : list) {
                InternalLocaleBuilder.CaseInsensitiveChar o = new CaseInsensitiveChar(s2);
                if (!set.contains(o)) {
                    if (UnicodeLocaleExtension.isSingletonChar(o.value())) {
                        this.setUnicodeLocaleExtension(s2.substring(2));
                    }
                    else {
                        if (this.extensions == null) {
                            this.extensions = new HashMap(4);
                        }
                        this.extensions.put(o, s2.substring(2));
                    }
                }
                set.add(o);
            }
        }
        if (s != null && s.length() > 0) {
            if (this.extensions == null) {
                this.extensions = new HashMap(1);
            }
            this.extensions.put(new CaseInsensitiveChar(s), s.substring(2));
            return this;
        }
        else {
            return this;
        }
    }
    
    public InternalLocaleBuilder setLanguageTag(LanguageTag languageTag) {
        this.clear();
        if (!languageTag.getExtlangs().isEmpty()) {
            this.language = (String)languageTag.getExtlangs().get(0);
        }
        else {
            String language = languageTag.getLanguage();
            if (!language.equals("und")) {
                this.language = language;
            }
        }
        this.script = languageTag.getScript();
        this.region = languageTag.getRegion();
        List<String> variants = languageTag.getVariants();
        if (!variants.isEmpty()) {
            StringBuilder sb = new StringBuilder((String)variants.get(0));
            for (int size = variants.size(), i = 1; i < size; ++i) {
                sb.append("_").append((String)variants.get(i));
            }
            this.variant = sb.toString();
        }
        this.setExtensions(languageTag.getExtensions(), languageTag.getPrivateuse());
        return this;
    }
    
    public InternalLocaleBuilder setLocale(BaseLocale baseLocale, LocaleExtensions localeExtensions) throws LocaleSyntaxException {
        String language = baseLocale.getLanguage();
        String script = baseLocale.getScript();
        String region = baseLocale.getRegion();
        String variant = baseLocale.getVariant();
        if (language.equals("ja") && region.equals("JP") && variant.equals("JP")) {
            assert !(!"japanese".equals(localeExtensions.getUnicodeLocaleType("ca")));
            variant = "";
        }
        else if (language.equals("th") && region.equals("TH") && variant.equals("TH")) {
            assert !(!"thai".equals(localeExtensions.getUnicodeLocaleType("nu")));
            variant = "";
        }
        else if (language.equals("no") && region.equals("NO") && variant.equals("NY")) {
            language = "nn";
            variant = "";
        }
        if (language.length() > 0 && !LanguageTag.isLanguage(language)) {
            throw new LocaleSyntaxException("Ill-formed language: " + language);
        }
        if (script.length() > 0 && !LanguageTag.isScript(script)) {
            throw new LocaleSyntaxException("Ill-formed script: " + script);
        }
        if (region.length() > 0 && !LanguageTag.isRegion(region)) {
            throw new LocaleSyntaxException("Ill-formed region: " + region);
        }
        if (variant.length() > 0) {
            int checkVariants = this.checkVariants(variant, "_");
            if (checkVariants != -1) {
                throw new LocaleSyntaxException("Ill-formed variant: " + variant, checkVariants);
            }
        }
        this.language = language;
        this.script = script;
        this.region = region;
        this.variant = variant;
        this.clearExtensions();
        Set<Object> set = (localeExtensions == null) ? null : localeExtensions.getKeys();
        if (set != null) {
            for (Character c : set) {
                Extension extension = localeExtensions.getExtension(c);
                if ((extension instanceof UnicodeLocaleExtension)) {
                    UnicodeLocaleExtension unicodeLocaleExtension = (UnicodeLocaleExtension)extension;
                    for (String s : unicodeLocaleExtension.getUnicodeLocaleAttributes()) {
                        if (this.uattributes == null) {
                            this.uattributes = new HashSet(4);
                        }
                        this.uattributes.add(new CaseInsensitiveString(s));
                    }
                    for (String s2 : unicodeLocaleExtension.getUnicodeLocaleKeys()) {
                        if (this.ukeywords == null) {
                            this.ukeywords = new HashMap(4);
                        }
                        this.ukeywords.put(new CaseInsensitiveString(s2), unicodeLocaleExtension.getUnicodeLocaleType(s2));
                    }
                }
                else {
                    if (this.extensions == null) {
                        this.extensions = new HashMap(4);
                    }
                    this.extensions.put(new InternalLocaleBuilder.CaseInsensitiveChar((char)c), extension.getValue());
                }
            }
            return this;
        }
        return this;
    }
    
    public InternalLocaleBuilder clear() {
        this.language = "";
        this.script = "";
        this.region = "";
        this.variant = "";
        this.clearExtensions();
        return this;
    }
    
    public InternalLocaleBuilder clearExtensions() {
        if (this.extensions != null) {
            this.extensions.clear();
        }
        if (this.uattributes != null) {
            this.uattributes.clear();
        }
        if (this.ukeywords != null) {
            this.ukeywords.clear();
            return this;
        }
        return this;
    }
    
    public BaseLocale getBaseLocale() {
        String language = this.language;
        String script = this.script;
        String region = this.region;
        String str = this.variant;
        if (this.extensions != null) {
            String s = (String)this.extensions.get(InternalLocaleBuilder.PRIVATEUSE_KEY);
            if (s != null) {
                StringTokenIterator stringTokenIterator = new StringTokenIterator(s, "-");
                int n = 0;
                int currentStart = -1;
                while (!stringTokenIterator.isDone()) {
                    if (n != 0) {
                        currentStart = stringTokenIterator.currentStart();
                        break;
                    }
                    if (LocaleUtils.caseIgnoreMatch(stringTokenIterator.current(), "lvariant")) {
                        n = 1;
                    }
                    stringTokenIterator.next();
                }
                if (currentStart != -1) {
                    StringBuilder o = new StringBuilder(str);
                    if (o.length() != 0) {
                        o.append("_");
                    }
                    o.append(s.substring(currentStart).replaceAll("-", "_"));
                    str = o.toString();
                }
            }
        }
        return BaseLocale.getInstance(language, script, region, str);
    }
    
    public LocaleExtensions getLocaleExtensions() {
        if (LocaleUtils.isEmpty(this.extensions) && LocaleUtils.isEmpty(this.uattributes) && LocaleUtils.isEmpty(this.ukeywords)) {
            return null;
        }
        LocaleExtensions localeExtensions = new LocaleExtensions(this.extensions, this.uattributes, this.ukeywords);
        return localeExtensions.isEmpty() ? null : localeExtensions;
    }
    
    static String removePrivateuseVariant(String s) {
        StringTokenIterator stringTokenIterator = new StringTokenIterator(s, "-");
        int currentStart = -1;
        boolean n = false;
        while (!stringTokenIterator.isDone()) {
            if (currentStart != -1) {
                n = true;
                break;
            }
            if (LocaleUtils.caseIgnoreMatch(stringTokenIterator.current(), "lvariant")) {
                currentStart = stringTokenIterator.currentStart();
            }
            stringTokenIterator.next();
        }
        if (n == 0) {
            return s;
        }
        assert currentStart > 1;
        return (currentStart == 0) ? null : s.substring(0, currentStart - 1);
    }
    
    private int checkVariants(String s, String s2) {
        StringTokenIterator stringTokenIterator = new StringTokenIterator(s, s2);
        while (!stringTokenIterator.isDone()) {
            if (!LanguageTag.isVariant(stringTokenIterator.current())) {
                return stringTokenIterator.currentStart();
            }
            stringTokenIterator.next();
        }
        return -1;
    }
    
    private void setUnicodeLocaleExtension(String s) {
        if (this.uattributes != null) {
            this.uattributes.clear();
        }
        if (this.ukeywords != null) {
            this.ukeywords.clear();
        }
        StringTokenIterator stringTokenIterator = new StringTokenIterator(s, "-");
        while (!stringTokenIterator.isDone() && UnicodeLocaleExtension.isAttribute(stringTokenIterator.current())) {
            if (this.uattributes == null) {
                this.uattributes = new HashSet(4);
            }
            this.uattributes.add(new InternalLocaleBuilder.CaseInsensitiveString(stringTokenIterator.current()));
            stringTokenIterator.next();
        }
        Object o = null;
        int currentStart = -1;
        int currentEnd = -1;
        while (!stringTokenIterator.isDone()) {
            if (o != null) {
                if (UnicodeLocaleExtension.isKey(stringTokenIterator.current())) {
                    assert currentEnd != -1;
                    String s2 = (currentStart == -1) ? "" : s.substring(currentStart, currentEnd);
                    if (this.ukeywords == null) {
                        this.ukeywords = new HashMap(4);
                    }
                    this.ukeywords.put(o, s2);
                    InternalLocaleBuilder.CaseInsensitiveString o2 = new InternalLocaleBuilder.CaseInsensitiveString(stringTokenIterator.current());
                    o = (this.ukeywords.containsKey(o2) ? null : o2);
                    currentEnd = (currentStart = -1);
                }
                else {
                    if (currentStart == -1) {
                        currentStart = stringTokenIterator.currentStart();
                    }
                    currentEnd = stringTokenIterator.currentEnd();
                }
            }
            else if (UnicodeLocaleExtension.isKey(stringTokenIterator.current())) {
                o = new InternalLocaleBuilder.CaseInsensitiveString(stringTokenIterator.current());
                if (this.ukeywords != null && this.ukeywords.containsKey(o)) {
                    o = null;
                }
            }
            if (!stringTokenIterator.hasNext()) {
                if (o != null) {
                    assert currentEnd != -1;
                    String s3 = (currentStart == -1) ? "" : s.substring(currentStart, currentEnd);
                    if (this.ukeywords == null) {
                        this.ukeywords = new HashMap(4);
                    }
                    this.ukeywords.put(o, s3);
                    break;
                }
                else {
                    break;
                }
            }
            else {
                stringTokenIterator.next();
            }
        }
    }
    
    static {
        PRIVATEUSE_KEY = new InternalLocaleBuilder.CaseInsensitiveChar("x");
    }
    
    static final class CaseInsensitiveString
    {
        private final String str;
        private final String lowerStr;
        
        CaseInsensitiveString(String str) {
            this.str = str;
            this.lowerStr = LocaleUtils.toLowerString(str);
        }
        
        public String value() {
            return this.str;
        }
        
        @Override
        public int hashCode() {
            return this.lowerStr.hashCode();
        }
        
        @Override
        public boolean equals(Object o) {
            return this == o || (o instanceof InternalLocaleBuilder.CaseInsensitiveString && this.lowerStr.equals(((InternalLocaleBuilder.CaseInsensitiveString)o).lowerStr));
        }
    }
    
    static final class CaseInsensitiveChar
    {
        private final char ch;
        private final char lowerCh;
        
        private CaseInsensitiveChar(String s) {
            this(s.charAt(0));
        }
        
        CaseInsensitiveChar(char ch) {
            this.ch = ch;
            this.lowerCh = LocaleUtils.toLower(this.ch);
        }
        
        public char value() {
            return this.ch;
        }
        
        @Override
        public int hashCode() {
            return this.lowerCh;
        }
        
        @Override
        public boolean equals(Object o) {
            return this == o || (o instanceof InternalLocaleBuilder.CaseInsensitiveChar && this.lowerCh == ((InternalLocaleBuilder.CaseInsensitiveChar)o).lowerCh);
        }
    }
}
